﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      The Cy4NET System                      Version 19.09                            //
//      __________________________             _________________________________        //
//                                                                                      //
//      Decentralised Realtime                 Copyright (c) 2012 Reimar Grasbon        //
//      Peer-to-peer Communication             (www.cy4net.org)                         //
//      Protocol                                                                        //
//      _________________________              _________________________________        //
//                                                                                      //
//      [Message]                                                                       //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Cy4NET-Nachricht                                                                    //
//  ----------------                                                                    //
//                                                                                      //
//                                                                                      //
//                    +-----+  +-----+  +-----+  +-----+  +---+---+-···-+---+---+       //
//      Cy4Message:   | ACC |  | ADR |  | LOC |  | CNT |  |         DAT         |       //
//                    +-----+  +-----+  +-----+  +-----+  +---+---+-···-+---+---+       //
//                                                                                      //
//                                                                                      //
//      · ACC:  Zugriffsart der Nachricht. <ACC> ist eine Ausprägung des Aufzählungs-   //
//              typs 'Cy4MessageAccess'. Mögliche Werte sind:                           //
//                                                                                      //
//                  · CY4MESSAGE_READ:  Lesenachricht, Daten werden abgerufen           //
//                  · CY4MESSAGE_WRITE: Schreibnachricht, Daten werden versendet        //
//                                                                                      //
//      · ADR:  Zieladresse der Nachricht als 'Cy4Address' (siehe dort).                //
//                                                                                      //
//      · LOC:  Dienstadresse. LOC ist eine vorzeichenlose 24-Bit-Zahl im Bereich von   //
//              0 bis FFFFFEh, FFFFFFh ist nicht erlaubt. Rundrufe sind festgelegt      //
//              auf den Adressbereich von E00000h bis EFFFFFh. Der Adressbereich        //
//              unter 100000h ist für Zugriffe des Programmladermoduls reserviert.      //
//                                                                                      //
//      · CNT:  Anzahl der Datenbytes in <DAT>. <CNT> kann Werte von 1 bis 64 annehmen. //
//                                                                                      //
//      · DAT:  <DAT> ist ein Feld, das Platz für <CNT> viele Datenbytes bietet. Je     //
//              nach Einsatz der Nachricht (als Anfrage oder als Antwort), wird hier    //
//              die übertragene Nutzlast bereitgestellt.                                //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#include "TypeDefs.h"
#include "Cy4Message.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void
CY4MESSAGE_SetupRead(Cy4Message* message,
                     Cy4Address  address,
                     UInt32      location,
                     UInt08      count)
{
    message->valid    = true;
    message->access   = CY4MESSAGE_READ;
    message->address  = address;
    message->location = location;
    message->count    = count;
}

//////////////////////////////////////////////////////////////////////////////////////////
void
CY4MESSAGE_SetupWrite(Cy4Message*   message,
                      Cy4Address    address,
                      UInt32        location,
                      const UInt08* data,
                      UInt08        count)
{
    message->valid    = true;
    message->access   = CY4MESSAGE_WRITE;
    message->address  = address;
    message->location = location;
    message->count    = count;
    for (UInt08 i=0; i<count; i++) message->data[i] = data[i];
}

//////////////////////////////////////////////////////////////////////////////////////////
void
CY4MESSAGE_Invalidate(Cy4Message* message)
{
    message->valid = false;
}

//////////////////////////////////////////////////////////////////////////////////////////
bool
CY4MESSAGE_IsValid(const Cy4Message* message)
{
    return message->valid;
}

//////////////////////////////////////////////////////////////////////////////////////////
Cy4MessageAccess
CY4MESSAGE_Access(const Cy4Message* message)
{
    return message->access;
}

//////////////////////////////////////////////////////////////////////////////////////////
Cy4Address
CY4MESSAGE_Address(const Cy4Message* message)
{
    return message->address;
}

//////////////////////////////////////////////////////////////////////////////////////////
UInt32
CY4MESSAGE_Location(const Cy4Message* message)
{
    return message->location;
}

//////////////////////////////////////////////////////////////////////////////////////////
UInt08
CY4MESSAGE_Count(const Cy4Message* message)
{
    return message->count;
}

//////////////////////////////////////////////////////////////////////////////////////////
const UInt08*
CY4MESSAGE_ConstData(const Cy4Message* message)
{
    return message->data;
}

//////////////////////////////////////////////////////////////////////////////////////////
UInt08*
CY4MESSAGE_Data(Cy4Message* message)
{
    return message->data;
}

//////////////////////////////////////////////////////////////////////////////////////////
